##############################################################################
#
# Copyright (c) 2002 Nexedi SARL and Contributors. All Rights Reserved.
#                    Jean-Paul Smets-Solane <jp@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowMethod

from Products.ERP5Type import Permissions, PropertySheet, Constraint, Interface
from Products.ERP5.Core import MetaNode, MetaResource

from Movement import Movement

class SimulationMovement(Movement):
  """
      Simulation movements belong to a simulation workflow which includes
      the following steps

      - planned

      - ordered

      - confirmed (the movement is now confirmed in qty or date)

      - started (the movement has started)

      - stopped (the movement is now finished)

      - delivered (the movement is now archived in a delivery)

      The simulation worklow uses some variables, which are
      set by the template

      - is_order_required

      - is_delivery_required


      XX
      - is_problem_checking_required ?

      Other flag
      (forzen flag)
  """
  meta_type = 'ERP5 Simulation Movement'
  portal_type = 'Simulation Movement'
  add_permission = Permissions.AddERP5Content
  isPortalContent = 1
  isRADContent = 1
  isMovement = 1

  # Declarative security
  security = ClassSecurityInfo()
  security.declareObjectProtected(Permissions.View)

  # Declarative interfaces
  __implements__ = ( Interface.Variated, )

  # Declarative properties
  property_sheets = ( PropertySheet.Base
                    , PropertySheet.SimpleItem
                    , PropertySheet.CategoryCore
                    , PropertySheet.Amount
                    , PropertySheet.Task
                    , PropertySheet.Arrow
                    , PropertySheet.Movement
                    , PropertySheet.Simulation
                    )

  # Factory Type Information
  factory_type_information = \
      {    'id'             : portal_type
         , 'meta_type'      : meta_type
         , 'description'    : """\
An Organisation object holds the information about
an organisation (ex. a division in a company, a company,
a service in a public administration)."""
         , 'icon'           : 'segment_icon.gif'
         , 'product'        : 'ERP5'
         , 'factory'        : 'addSimulationMovement'
         , 'immediate_view' : 'predicate_view'
         , 'actions'        :
        ( { 'id'            : 'view'
          , 'name'          : 'View'
          , 'category'      : 'object_view'
          , 'action'        : 'predicate_view'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'print'
          , 'name'          : 'Print'
          , 'category'      : 'object_print'
          , 'action'        : 'segment_print'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'metadata'
          , 'name'          : 'Metadata'
          , 'category'      : 'object_view'
          , 'action'        : 'metadata_edit'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'translate'
          , 'name'          : 'Translate'
          , 'category'      : 'object_action'
          , 'action'        : 'segment_view'
          , 'permissions'   : (
              Permissions.TranslateContent, )
          }
        )
      }

  security.declareProtected(Permissions.AccessContentsInformation, 'getCausalitySate')
  def getCausalityState(self, id_only=1):
    """
      Returns the current state in causality
    """
    portal_workflow = getToolByName(self, 'portal_workflow')
    wf = portal_workflow.getWorkflowById('causality_workflow')
    return wf._getWorkflowStateOf(self, id_only=id_only)

  security.declareProtected(Permissions.AccessContentsInformation, 'getSimulationSate')
  def getSimulationState(self, id_only=1):
    """
      Returns the current state in simulation
    """
    portal_workflow = getToolByName(self, 'portal_workflow')
    wf = portal_workflow.getWorkflowById('simulation_workflow')
    return wf._getWorkflowStateOf(self, id_only=id_only )

  #######################################################
  # Simulation Workflow Methods

  security.declareProtected(Permissions.ModifyPortalContent, 'plan')
  def plan(self):
    """
      Sets the movement to planned
      This should trigger the application or rules
    """

  plan = WorkflowMethod(plan)

  security.declareProtected(Permissions.ModifyPortalContent, 'order')
  def order(self):
    """
      Sets the movement to ordered

    """

  order = WorkflowMethod(order)

  security.declareProtected(Permissions.ModifyPortalContent, 'confirm')
  def confirm(self):
    """
      Sets the movement to confirmed
    """

  confirm = WorkflowMethod(confirm)

  security.declareProtected(Permissions.ModifyPortalContent, 'start')
  def start(self):
    """
      Starts the movement
      This should make the movement start_frozen
    """

  start = WorkflowMethod(start)

  security.declareProtected(Permissions.ModifyPortalContent, 'stop')
  def stop(self):
    """
      Stops the movement
    """

  stop = WorkflowMethod(stop)

  security.declareProtected(Permissions.ModifyPortalContent, 'deliver')
  def deliver(self):
    """
      Makes the movement delivered
    """

  deliver = WorkflowMethod(deliver)

  security.declareProtected(Permissions.ModifyPortalContent, 'cancel')
  def cancel(self):
    """
      Makes the movement cancelled
    """

  cancel = WorkflowMethod(cancel)

  # Acounting
  security.declareProtected(Permissions.AccessContentsInformation, 'isAccountable')
  def isAccountable(self):
    """
      Returns 1 if this needs to be accounted
    """

  # Ordering / Delivering
  security.declareProtected(Permissions.AccessContentsInformation, 'requiresOrder')
  def requiresOrder(self):
    """
      Returns 1 if this needs to be ordered
    """
    if isOrderable():
      return len(self.getCategoryMembership('order')) is 0
    else:
      return 0

  security.declareProtected(Permissions.AccessContentsInformation, 'requiresDelivery')
  def requiresDelivery(self):
    """
      Returns 1 if this needs to be accounted
    """
    if isDeliverable():
      return len(self.getCategoryMembership('delivery')) is 0
    else:
      return 0


  #######################################################
  # Causality Workflow Methods

  security.declareProtected(Permissions.ModifyPortalContent, 'expand')
  def expand(self):
    """
      -> new status : expanded

      Parses all existing applied rules and make sure they apply.
      Checks other possible rules and starts expansion process
      (instanciates rule and calls expand on rule)

      Only movements which applied rule parent is expanded can
      be expanded.
    """
    if self.getCausalityState() == 'expanded': return

    portal_rules = getToolByName(self, 'portal_rules')
    # Parse each applied rule and test if it applied
    #for applied_rule in self.objectValues():
    #  if not applied_rule.test():
    #    # delete
    # Parse each rule and test if it applies
    for rule in portal_rules.objectValues():
      if rule.test(self):
        my_applied_rule = rule.constructNewAppliedRule(self)
        my_applied_rule.expand()

  expand = WorkflowMethod(expand)

  security.declareProtected(Permissions.ModifyPortalContent, 'solve')
  def solve(self):
    """
       Makes the movement expandable again

       -> new status -> solved

       Once a movement has been updated with consistent
       target and planned values, it is marked as solved
       and can therefore be expanded again
    """
    if self.getCausalityState() == 'solved': return

  solve = WorkflowMethod(solve)

  security.declareProtected(Permissions.ModifyPortalContent, 'diverge')
  def diverge(self):
    """
       -> new status -> diverged

       Movements which diverge can not be expanded
    """
    if self.getCausalityState() == 'diverged': return

  diverge = WorkflowMethod(diverge)

  # Target computation
  security.declareProtected(Permissions.View, 'isDivergent')
  def isDivergent():
    """
      Returns 1 if the target is not met according to the current information
      After and edit, the isOutOfTarget will be checked. If it is 1,
      a message is emitted

      emit targetUnreachable !
    """


  #######################################################
  # Defer indexing process for now XXX

  def reindexObject2(self, *k, **kw):
    """
    """
